package net.sf.seaf.test.internal.unitils;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import org.unitils.reflectionassert.ReflectionComparator;
import org.unitils.reflectionassert.comparator.Comparator;
import org.unitils.reflectionassert.difference.Difference;

/**
 * A Unitils {@link Comparator} comparing {@link Date} and {@link Calendar}
 * values via ISO timestamp String.
 */
public class TemporalComparator implements Comparator {

	public boolean canCompare(Object left, Object right) {
		return isAllowed(left) && isAllowed(right);
	}

	private boolean isAllowed(Object o) {
		return (null == o) || (o instanceof Date) || (o instanceof Calendar);
	}

	public Difference compare(Object left, Object right,
			boolean onlyFirstDifference,
			ReflectionComparator reflectionComparator) {
		String sLeft = asString(left);
		String sRight = asString(right);
		if (!sLeft.equals(sRight)) {
			return new Difference("Different temporal values", sLeft, sRight);
		}
		return null;
	}

	private String asString(Object o) {
		if (null == o) {
			return String.valueOf(o);
		}
		SimpleDateFormat f = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSZ");
		if (o instanceof Calendar) {
			Calendar cal = (Calendar) o;
			return f.format(cal.getTime());
		}
		if (o instanceof Date) {
			return f.format((Date) o);
		}
		throw new IllegalArgumentException("Unsupported type: "
				+ o.getClass().getName());
	}

}
