package net.sf.seaf.test.jmock;

import net.sf.seaf.test.jmock.matchers.JMockDeepEquals;

import org.hamcrest.Matcher;

/**
 * SEAF matchers for JMock tests.
 * <p>
 * See {@link Matcher} for general info about matchers.
 */
public class JMockSeafMatchers {

	/**
	 * Matcher that checks deep reflection equality to the given value.
	 * <p>
	 * This matcher can be used when equals() is not implemented on compared
	 * objects. Matcher uses java reflection API to compare fields of wanted and
	 * actual object.
	 * <p>
	 * Works similarly to {@link org.unitils.reflectionassert.ReflectionAssert}
	 * .assertReflectionEquals() from unitils.org.
	 * <p>
	 * This is a deep compare, the reflection algorithm delves recursively into
	 * the properties of the compared beans.
	 * 
	 * @param value
	 *            Value to compare
	 * @return <code>null</code>.
	 */
	public static <T> Matcher<T> deepEq(T value) {
		return new JMockDeepEquals<T>(value);
	}

}
