package net.sf.seaf.test.mockito;

import net.sf.seaf.test.mockito.matchers.MockitoDeepEquals;

import org.mockito.Matchers;

/**
 * SEAF matchers for Mockito tests.
 * <p>
 * See {@link Matchers} for general info about matchers.
 */
@SuppressWarnings("unchecked")
public class MockitoSeafMatchers {

	/**
	 * Object argument that is deep-reflection-equal to the given value.
	 * <p>
	 * This matcher can be used when equals() is not implemented on compared
	 * objects. Matcher uses java reflection API to compare fields of wanted and
	 * actual object.
	 * <p>
	 * Works similarly to {@link org.unitils.reflectionassert.ReflectionAssert}
	 * .assertReflectionEquals() from unitils.
	 * <p>
	 * This is a deep compare, the reflection algorithm delves recursively into
	 * the properties of the compared beans.
	 * 
	 * @param value
	 *            Value to compare
	 * @return <code>null</code>.
	 */
	public static <T> T deepEq(T value) {
		return (T) Matchers.argThat(new MockitoDeepEquals(value));
	}

}
