package net.sf.seaf.test.mockito.matchers;

import java.io.Serializable;

import net.sf.seaf.test.internal.unitils.SeafReflectionComparatorFactory;

import org.hamcrest.Description;
import org.hamcrest.SelfDescribing;
import org.mockito.ArgumentMatcher;
import org.mockito.internal.matchers.ContainsExtraTypeInformation;
import org.unitils.reflectionassert.ReflectionComparator;
import org.unitils.reflectionassert.difference.Difference;

public class MockitoDeepEquals extends ArgumentMatcher<Object> implements
		ContainsExtraTypeInformation, Serializable {

	private static final long serialVersionUID = 6091161448661578340L;
	private final Object expected;

	public MockitoDeepEquals(Object expected) {
		this.expected = expected;
	}

	@Override
	public boolean matches(Object actual) {
		ReflectionComparator reflectionComparator = SeafReflectionComparatorFactory
				.createRefectionComparator();
		Difference difference = reflectionComparator.getDifference(expected,
				actual);
		return null == difference;
	}

	@Override
	public void describeTo(Description description) {
		description.appendText(describe(expected));
	}

	public String describe(Object object) {
		String text = quoting();
		if (object == null) {
			text += "null";
		} else {
			text += object.toString();
		}
		text += quoting();
		return text;
	}

	private String quoting() {
		if (expected instanceof String) {
			return "\"";
		} else if (expected instanceof Character) {
			return "'";
		} else {
			return "";
		}
	}

	public SelfDescribing withExtraTypeInfo() {
		return new SelfDescribing() {
			public void describeTo(Description description) {
				description
						.appendText(describe("("
								+ expected.getClass().getSimpleName() + ") "
								+ expected));
			}
		};
	}

	public boolean typeMatches(Object object) {
		return expected != null && object != null
				&& object.getClass() == expected.getClass();
	}

}
