package net.sf.seaf.test.unitils;

import java.util.Calendar;

import junit.framework.Assert;
import net.sf.seaf.test.internal.unitils.SeafReflectionComparatorFactory;
import net.sf.seaf.test.internal.unitils.TemporalComparator;

import org.unitils.reflectionassert.ReflectionComparator;
import org.unitils.reflectionassert.difference.Difference;
import org.unitils.reflectionassert.report.impl.DefaultDifferenceReport;

/**
 * Performs deep equals using Unitils ReflectionComparator, using our own
 * {@link TemporalComparator} to take care of {@link Calendar} values correctly.
 */
public class DeepEquals {

	/**
	 * Perform deep equals using Unitils ReflectionComparator, using our own
	 * {@link TemporalComparator} to take care of {@link Calendar} values
	 * correctly.
	 * <p>
	 * Fail the test if values not matching.
	 * 
	 * @param expected
	 *            Expected value
	 * @param actual
	 *            Actual value
	 */
	public static void matches(Object expected, Object actual) {
		ReflectionComparator reflectionComparator = SeafReflectionComparatorFactory
				.createRefectionComparator();
		Difference difference = reflectionComparator.getDifference(expected,
				actual);
		if (difference != null) {
			Assert.fail(new DefaultDifferenceReport().createReport(difference));
		}
	}

}
