package net.sf.seaf.test.util;

import net.sf.seaf.test.internal.InternalTestDataCreator;

/**
 * Creates an instance of specified class, initialized with data values suitable
 * for various types of tests.
 */
public class TestDataCreator {

	/**
	 * Create an instance of the required type initialized with generated test
	 * data.
	 * <p>
	 * The test data are based on the simple class name of the required type and
	 * offset by 1.
	 * <p>
	 * Recurses to other beans accessible by setter. This can only be used for
	 * trees of beans, cyclic graphs produce stack overflow.
	 * 
	 * @param type
	 *            The type to instantiate and initialize
	 * @return An initialized instance of the required type
	 */
	public <T> T create(Class<T> type) {
		return create(type, type.getSimpleName(), 1);
	}

	/**
	 * Create an instance of the required type initialized with generated test
	 * data.
	 * <p>
	 * The test data are based on the simple class name of the required type and
	 * offset by 1.
	 * 
	 * @param type
	 *            The type to instantiate and initialize
	 * @param recurse
	 *            Recurse to other beans accessible by setter? This can only be
	 *            used for trees of beans, cyclic graphs produce stack overflow.
	 * @return An initialized instance of the required type
	 */
	public <T> T create(Class<T> type, boolean recurse) {
		return create(type, type.getSimpleName(), 1, recurse);
	}

	/**
	 * Create an instance of the required type initialized with generated test
	 * data. The test data values are based on the baseValue and offset by the
	 * index.
	 * <p>
	 * Recurses to other beans accessible by setter. This can only be used for
	 * trees of beans, cyclic graphs produce stack overflow.
	 * 
	 * @param type
	 *            The type to instantiate and initialize
	 * @param baseValue
	 *            The base value for test data
	 * @param index
	 *            The offset for the test data
	 * @return An initialized instance of the required type
	 */
	public <T> T create(Class<T> type, Object baseValue, int index) {
		return create(type, baseValue, index, true);
	}

	/**
	 * Create an instance of the required type initialized with generated test
	 * data. The test data values are based on the baseValue and offset by the
	 * index.
	 * 
	 * @param type
	 *            The type to instantiate and initialize
	 * @param baseValue
	 *            The base value for test data
	 * @param index
	 *            The offset for the test data
	 * @param recurse
	 *            Recurse to other beans accessible by setter? This can only be
	 *            used for trees of beans, cyclic graphs produce stack overflow.
	 * @return An initialized instance of the required type
	 */
	public <T> T create(Class<T> type, Object baseValue, int index,
			boolean recurse) {
		return new InternalTestDataCreator(recurse).create(type, baseValue,
				index);
	}
}
